// Copyright (C) 2000 Nicholas Schmitt nick@math.umass.edu

#ifndef Math_INCLUDED
#define Math_INCLUDED

namespace base {

class Math
{

public:

  /// the number with absolute value x and sign of s.
  template <class A> static inline A adjust_sign( A, A );

  /// maximum of two numbers.
  template <class A> static inline A max( A, A );

  /// maximum of three numbers.
  template <class A> static inline A max( A, A, A );

  /// minimum of two numbers.
  template <class A> static inline A min( A, A );

  /// minimum of three numbers.
  template <class A> static inline A min( A, A, A );

  /// 
  template <class A> static inline A sqr( A );

  /// the cube of a number
  template <class A> static inline A cube( A );

  /// the average of two numbers.
  template <class A> static inline A midpoint( A, A );
#if 0
  ///
  template <class A> static inline A round( A );

  ///
  template <class A> static inline A round( A, int );
#endif
  ///
  template <class A> static inline bool between( A, A, A );

  ///
  template <class A> static inline bool distinct( A, A, A );

  ///
  template <class A> static inline bool same_sign( A, A );

  ///
  template <class A> static inline int sign( A );

  ///
  template <class A> static inline void swap( A &, A & );

  ///
  template <class A> static inline void sort( A &, A & );

  ///
  template <class A> static inline A mod( A x, A n );
protected:

private:

};



template <class A>
A
Math::
max( A a, A b )
{
  return a >= b ? a : b;
}


template <class A>
A
Math::
min( A a, A b )
{
  return a <= b ? a : b;
}


template <class A>
A
Math::
max( A a, A b, A c )
{
  return a > b ? Math::max(c,a) : Math::max(c,b);
}


template <class A>
A
Math::
min( A a, A b, A c )
{
  return b < a ? Math::min(c,b) : Math::min(c,a);
}


template <class A>
A
Math::
sqr( A x )
{
  return x*x;
}


template <class A>
A
Math::
cube( A x )
{
  return x*x*x;
}


template <class A>
int
Math::
sign( A a )
{
  return ( a > 0 ? 1 : ( a < 0 ? -1 : 0 ) );
}


template <class A>
bool
Math::
between( A a, A b, A c )
{
  return ( a < b && b < c );
}


template <class A>
bool
Math::
distinct( A a, A b, A c )
{
  return ( a != b && a != c && b != c );
}


template <class A>
bool
Math::
same_sign( A a, A b )
{
  return ( ( a > 0.0 && b > 0.0 ) || ( a < 0.0 && b < 0.0 ) ||
           ( a == 0.0 && b == 0.0 ) );
}


template <class A>
A
Math::
midpoint( A a, A b )
{
  return 0.5 * ( a + b );
}


template <class A>
A
Math::
adjust_sign( A x, A s )
{
  assert( s != 0.0 );
  return( s > 0.0 ? fabs(x) : -fabs(x) );
}

#if 0
template <class A>
A
Math::
round( A x )
{
  A r;
  if ( x > 0.0 )
  {
    r = floor(x + 0.5);
  }
  else if ( x < 0.0 )
  {
    r = ceil(x - 0.5);
  }
  else
  {
    r = 0;
  }
  return r;
}


template <class A>
A
Math::
round( A x, int i )
{
  double y = pow(10.0, double(i));
  return round( x * y ) / y;
}
#endif

template <class A>
void
Math::
swap( A &a, A &b )
{
  A tmp = a;
  a = b;
  b = tmp;
}


template <class A>
void
Math::
sort( A &a, A &b )
{
  if ( a > b ) { swap( a, b ); }
}


template <class A>
A
Math::
mod( A x, A n )
{
  A y = x % n;
  if (y < 0) { y += n; }
  return y;
}

} // namespace base

#endif // !Math_INCLUDED
