// Copyright (C) 2000 Nicholas Schmitt nick@math.umass.edu

#include "MathReal.H"

namespace math {
using namespace base;

#ifdef M_PI
double const MathReal::pi = M_PI;
#else
double const MathReal::pi = acos(-1.0);
#endif

/*
  -----------------------------------------------------------
  formatted print
  */

void
MathReal::
print( ostream &o, float x )
{
  int unsigned const precision_ = 8;
  
  // save current state
  long int flags = o.flags();
  int long precision = o.precision();
  o.precision(precision_);
  o.setf(ios::scientific, ios::floatfield);
  o.width(precision_+7);

  // print
  o << x;

  // restore original state
  o.precision( (int)precision );
  o.flags( (int unsigned)flags );
}


void
MathReal::
print( ostream &o, double x )
{
  int unsigned const precision_ = 15;

  // save current state
  long int flags = o.flags();
  int long precision = o.precision();
  o.precision(precision_);
  o.setf(ios::scientific, ios::floatfield);
  o.width(precision_+7);

  // print
  o << x;

  // restore original state
  o.precision( (int)precision );
  o.flags( (int unsigned)flags );
}


void
MathReal::
print( ostream &o, double long x )
{
  int unsigned const precision_ = 30;
  
  // save current state
  long int flags = o.flags();
  int long precision = o.precision();
  o.precision(precision_);
  o.setf(ios::scientific, ios::floatfield);
  o.width(precision_+7);

  // print
  o << x;

  // restore original state
  o.precision( (int)precision );
  o.flags( (int unsigned)flags );
}


void
MathReal::
printM( ostream &o, float x )
{
  int unsigned const precision_ = 8;
  // print in form "d.ddddddd*10^(+dd)
  
  // save current state
  long int flags = o.flags();
  int long precision = o.precision();
  o.precision(precision_);
  o.setf(ios::fixed, ios::floatfield);
  o.setf(ios::showpos);
  o.width(precision_+3);

  int exponent = 0;
  double mantissa = 0;
  
  if ( x != 0.0 )
  {
    exponent = (int)floor(log10(fabs(x)));
    mantissa = x/pow(double(10.0), double(exponent));
  }
  o << mantissa << "*10^(" << exponent << ")";

  // restore original state
  o.precision( (int)precision );
  o.flags( (int unsigned)flags );
}


void
MathReal::
printM( ostream &o, double x )
{
  int unsigned const precision_ = 15;
  // print in form "d.ddddddd*10^(+dd)
  
  // save current state
  long int flags = o.flags();
  int long precision = o.precision();
  o.precision(precision_);
  o.setf(ios::fixed, ios::floatfield);
  o.setf(ios::showpos);
  o.width(precision_+3);

  int exponent = 0;
  double mantissa = 0;
  
  if ( x != 0.0 )
  {
    exponent = (int)floor(log10(fabs(x)));
    mantissa = x/pow(double(10.0), double(exponent));
  }
  o << mantissa << "*10^(" << exponent << ")";

  // restore original state
  o.precision( (int)precision );
  o.flags( (int unsigned)flags );
}


void
MathReal::
printM( ostream &o, double long x )
{
  int unsigned const precision_ = 30;
  // print in form "d.ddddddd*10^(+dd)
  
  // save current state
  long int flags = o.flags();
  int long precision = o.precision();
  o.precision(precision_);
  o.setf(ios::fixed, ios::floatfield);
  o.setf(ios::showpos);
  o.width(precision_+3);

  int exponent = 0;
  double mantissa = 0;
  
  if ( x != 0.0 )
  {
    exponent = (int)floor(log10(fabs(x)));
    mantissa = x/pow(double(10.0), double(exponent));
  }
  o << mantissa << "*10^(" << exponent << ")";

  // restore original state
  o.precision( (int)precision );
  o.flags( (int unsigned)flags );
}


// factorial
// this function is not thread-safe

double
MathReal::
factorial( int unsigned n )
{
  static double const fact[] = {
    1.0000000000000000e00,  // 0!
    1.0000000000000000e00,
    2.0000000000000000e00,
    6.0000000000000000e00,
    2.4000000000000000e01,
    1.2000000000000000e02,
    7.2000000000000000e02,
    5.0400000000000000e03,
    4.0320000000000000e04,
    3.6288000000000000e05,
    3.6288000000000000e06,
    3.9916800000000000e07,
    4.7900160000000000e08,
    6.2270208000000000e09,
    8.7178291200000000e10,
    1.3076743680000000e12,
    2.0922789888000000e13,
    3.5568742809600000e14,
    6.4023737057280000e15,
    1.2164510040883200e17,
    2.4329020081766400e18,
    5.1090942171709440e19,
    1.1240007277776077e21,
    2.5852016738884977e22,
    6.2044840173323944e23,
    1.5511210043330986e25  // 25!
  };
  double const FACTORIAL_OVERFLOW = -1.0;
  USE(FACTORIAL_OVERFLOW);
  
  if ( n <= 25 ) { return fact[n]; }

  double y = gamma( double(n)+1 );

#if defined(LN_MAXDOUBLE)
  return ( y > LN_MAXDOUBLE ? FACTORIAL_OVERFLOW : signgam * exp(y) );
#else
  return signgam * exp(y);
#endif
}

} // namespace math
