// Copyright (C) 2000 Nicholas Schmitt nick@math.umass.edu

#include <ctype.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include "Math.H"
#include "String.H"

namespace base {

void
string::
uppercase( string const &s )
{
  int unsigned i;
  for ( i = 0; i < s.length(); i++ )
  {
    (*this)[i] = toupper(s[i]);
  }
}

void
string::
lowercase( string const &s )
{
  int unsigned i;
  for ( i = 0; i < s.length(); i++ )
  {
    (*this)[i] = tolower(s[i]);
  }
}

void
string::
uppercase()
{
  int unsigned i;
  for ( i = 0; i < length(); i++ )
  {
    (*this)[i] = toupper( (*this)[i]);
  }
}

void
string::
lowercase()
{
  int unsigned i;
  for ( i = 0; i < length(); i++ )
  {
    (*this)[i] = tolower((*this)[i]);
  }
}

int
string::
compare_nocase(string const &a, string const &b)
{
  size_t i, n;
  int c = 0;

  n = Math::min(a.length(), b.length());
    
  for (i = 0; i < n; i++)
  {
    c = int(toupper(a[i])) - int(toupper(b[i]));
    if (c != 0) { break; }
  }

  return (int)(i < n ? Math::sign(c) :
               Math::sign(int(a.length()) - int(b.length())));
}

int
string::
find_nocase(vector<string> const &s, string const &pattern)
{
  unsigned i;
  for (i = 0; i < s.size(); i++)
  {
    if (!compare_nocase(s[i], pattern))
       break;
  }
  return (i < s.size() ? (int)i : -1);
}

void
string::
strip_front( char c )
{
  unsigned n = 0;

  while ( n < length() && (*this)[n] == c) { n++; }

  *this = this->substr( n, length() - n );
}

void
string::
strip_back( char c )
{
  unsigned n = 0;

  while (n < length() && (*this)[length() - n - 1] == c) { n++; }

  *this = this->substr( 0, length() - n );
}

void
string::
strip( char c )
{
  strip_front( c );
  strip_back( c );
}

int
string::
read( istream &i, char marker )
{
  clear();
  if ( !i ) { return 1; }

  for (;;)
  {
    int n = i.get();
    if ( n == EOF || !i ) { return length() == 0; }
    if ( char(n) == marker ) { return 0; }
    *this += char(n);
  }
}

int
string::
split( vector<string> &w )
{
  size_t i = 0;
  size_t n = length();
  size_t m;

  w.erase(w.begin(), w.end());

  do
  {
    for (;;)
    {
      if (i >= n) { break; }
      if (!isspace( (*this)[i] )) { break; }
      i++;
    }

    if (i < n)
    {
      m = i;
      i++;

      for (;;)
      {
	if (i >= n) { break; }
	if (isspace( (*this)[i] )) { break; }
	i++;
      }

      w.push_back( substr(m, i - m) );
    }

  }
  while (i < n);

  return (int)w.size();
}

int
string::
to_int( int &x ) const
{
  int err = 0;
  errno = 0;
  char *c_str = allocate();
  x = (int)strtol(c_str, (char **)NULL, 10);
  if ( errno != 0 )
  {
    err = 1;
    x = (int)0;
  }
  deallocate( c_str );
  return err;
}

int
string::
to_int( int unsigned &x ) const
{
  int err = 0;
  errno = 0;
  char *c_str = allocate();
  x = (int unsigned)strtoul(c_str, (char **)NULL, 10);
  if ( errno != 0 )
  {
    err = 1;
    x = (int unsigned)0;
  }
  deallocate( c_str );
  return err;
}

int
string::
to_int( int short &x ) const
{
  int err = 0;
  errno = 0;
  char *c_str = allocate();
  x = (int short)strtol(c_str, (char **)NULL, 10);
  if ( errno != 0 )
  {
    err = 1;
    x = (int short)0;
  }
  deallocate( c_str );
  return err;
}

int
string::
to_int( int unsigned short &x ) const
{
  int err = 0;
  errno = 0;
  char *c_str = allocate();
  x = (int unsigned short)strtoul(c_str, (char **)NULL, 10);
  if ( errno != 0 )
  {
    err = 1;
    x = (int unsigned short)0;
  }
  deallocate( c_str );
  return err;
}

int
string::
to_int( int long &x ) const
{
  int err = 0;
  errno = 0;
  char *c_str = allocate();
  x = (int long)strtol(c_str, (char **)NULL, 10);
  if ( errno != 0 )
  {
    err = 1;
    x = (int long)0;
  }
  deallocate( c_str );
  return err;
}

int
string::
to_int( int unsigned long &x ) const
{
  int err = 0;
  errno = 0;
  char *c_str = allocate();
  x = (int unsigned long)strtoul(c_str, (char **)NULL, 10);
  if ( errno != 0 )
  {
    err = 1;
    x = (int unsigned long)0;
  }
  deallocate( c_str );
  return err;
}

int
string::
to_real( float &x ) const
{
  int err = 0;
  errno = 0;
  char *c_str = allocate();
  x = (float)strtod(c_str, (char **)NULL);
  if ( errno != 0 )
  {
    err = 1;
    x = (float)0.0;
  }
  deallocate( c_str );
  return err;
}

int
string::
to_real( double &x ) const
{
  int err = 0;
  errno = 0;
  char *c_str = allocate();
  x = (double)strtod(c_str, (char **)NULL);
  if ( errno != 0 )
  {
    err = 1;
    x = (double)0.0;
  }
  deallocate( c_str );
  return err;
}

int
string::
to_real( double long &x ) const
{
  int err = 0;
  errno = 0;
  char *c_str = allocate();
  x = (double long)strtod(c_str, (char **)NULL);
  if ( errno != 0 )
  {
    err = 1;
    x = (double long)0.0;
  }
  deallocate( c_str );
  return err;
}

void
string::
to_string( int x )
{
  char c[128]; // can this overflow ???
  sprintf( c, "%d", x );
  *this = c;
}

void
string::
to_string( int long x )
{
  char c[128]; // can this overflow ???
  sprintf( c, "%ld", x );
  *this = c;
}

void
string::
to_string( int unsigned x )
{
  char c[128]; // can this overflow ???
  sprintf( c, "%u", x );
  *this = c;
}

void
string::
to_string( int unsigned long x )
{
  char c[128]; // can this overflow ???
  sprintf( c, "%lu", x );
  *this = c;
}

void
string::
to_string( float x )
{
  char c[128]; // can this overflow ???
  sprintf( c, "%g", x );  // can this overflow with 10000000000000000000???
  *this = c;
}

void
string::
to_string( double x )
{
  char c[128]; // can this overflow ???
  sprintf( c, "%g", x );  // can this overflow???
  *this = c;
}

void
string::
to_string( double long x )
{
  char c[128]; // can this overflow ???
  sprintf( c, "%Lg", x );  // can this overflow???
  *this = c;
}

void
string::
to_string( complex<float> const &z )
{
  clear();
  
  float re = real(z);
  float im = imag(z);
  bool flag= false;

  if ( z == complex<float>(0.0) )
  {
    *this = "0";
    goto end;
  }

  if ( re != 0 )
  {
    to_string( re );
    flag = true;
  }
  
  if ( im == 1.0 )
  {
    *this += ( flag ? "+i" : "i" );
  }
  else if ( im == -1 )
  {
    *this += ( flag ? "+i" : "-i" );
  }
  else if ( im != 0.0 )
  {
    string t;
    t.to_string( im );
    if ( im > 0.0 && flag ) { *this += "+"; }
    *this += t;
    *this += "i";
  }
  end:
  ;
}

void
string::
to_string( complex<double> const &z )
{
  clear();
  
  double re = real(z);
  double im = imag(z);
  bool flag= false;

  if ( z == complex<double>(0.0) )
  {
    *this = "0";
    goto end;
  }

  if ( re != 0 )
  {
    to_string( re );
    flag = true;
  }
  
  if ( im == 1.0 )
  {
    *this += ( flag ? "+i" : "i" );
  }
  else if ( im == -1 )
  {
    *this += ( flag ? "+i" : "-i" );
  }
  else if ( im != 0.0 )
  {
    string t;
    t.to_string( im );
    if ( im > 0.0 && flag ) { *this += "+"; }
    *this += t;
    *this += "i";
  }
  end:
  ;
}

void
string::
to_string( complex<double long> const &z )
{
  clear();
  
  double re = real(z);
  double im = imag(z);
  bool flag= false;

  if ( z == complex<double long>(0.0) )
  {
    *this = "0";
    goto end;
  }

  if ( re != 0 )
  {
    to_string( re );
    flag = true;
  }
  
  if ( im == 1.0 )
  {
    *this += ( flag ? "+i" : "i" );
  }
  else if ( im == -1 )
  {
    *this += ( flag ? "+i" : "-i" );
  }
  else if ( im != 0.0 )
  {
    string t;
    t.to_string( im );
    if ( im > 0.0 && flag ) { *this += "+"; }
    *this += t;
    *this += "i";
  }
  end:
  ;
}

int
string::
read_word( istream& in )
{
  char c;

  clear();

  // eat initial whitespace
  for (;;)
  {
    in.get( c );
    if (!in.operator void*()) { goto end; }
    if ( !isspace( c ) ) { break; }
  }
  // read characters until next whitespace
  for (;;)
  {
    append(c);
    in.get( c );
    if ( !in.operator void*() ) { break; }
    if ( isspace( c ) ) { break; }
  }
  end:
  return ( !in.operator void*() && length() == 0 );  
}

int
string::
read_word(
  istream &in,
  unsigned &line_number,
  unsigned &column_number )
{
  char c;
  
  clear();

  // eat initial whitespace
  for ( ; ; )
  {
    in.get( c );
    if (!in.operator void*())
      goto end;
    
    if ( isspace( c ) )
    {
      if ( c == '\n' )
      {
        column_number = 0;
        line_number++;
      }      
    }
    else
      break;
  }

  // read characters until next whitespace
  for ( ; ; )
  {
    column_number++;
    append(c);

    in.get( c );
    if ( !in.operator void*() )
      break;

    if ( isspace( c ) )
    {
      if ( c == '\n' )
      {
        column_number = 0;
        line_number++;
      }
      break;
    }
  }
  
  end:
  return ( !in.operator void*() && length() == 0 );
}


char *
string::
allocate() const
{
  // allocate() returns new data that must be deleted with a call to
  // deallocate()
  char *c_str = new char[length()+1];
  memcpy( c_str, data(), length() );
  c_str[length()] = 0;
  return c_str;
}

void
string::
deallocate( char *c_str )
{
  delete [] c_str;
}

} // namespace base
