// Copyright (C) 2000 Nicholas Schmitt nick@math.umass.edu

#ifndef stringx_INCLUDED
#define stringx_INCLUDED

#include <vector.h>
#include "StringBase.H"
#include "Complex.H"

#if defined(__sgi) && !defined(__GNUG__)
#pragma set woff 1375 // destrctor not virtual
#endif

namespace base {

/**
  Extended functionality for string class.

  file stringx.H
*/
class string
  :
  public StringBase<char>
{
public:
  typedef char Char;

  /// construct a blank string
  string() :
      StringBase<Char>() {}

  /// construct a string equal to the length-n string s
  inline string( Char const s[], size_t n ) :
      StringBase<Char>( s, n ) {}

  /// construct a string equal to the 0-terminated string s
  inline string( Char const s[] ) :
      StringBase<Char>( s ) {}

  /// construct a string equal to the string s
  inline string( string const &s ) :
      StringBase<Char>( s ) {}

  // construct a string equal to length-n substring of s
  ///   with offset pos
  string( string const &s, size_t pos, size_t n ) :
      StringBase<Char>( s, pos, n ) {}

  /// construct a string equal to a single element c
  string( Char c ) :
      StringBase<Char>( c ) {}

  /// construct a string filled with n copies of c
  string( size_t n, Char c ) :
      StringBase<Char>( n, c ) {}

  /// construct
  string( StringBase<Char> const &s ) :
      StringBase<Char>( s ) {}

  /// substr()
  string substr( size_t pos, size_t n ) const
    { return string( *this, pos, n ); }

  /// destructor
  ~string() {};

  
  /// *this := uppercase(s)
  void uppercase( string const &s );

  /// *this := lowercase(s)
  void lowercase( string const &s );
  ///
  void uppercase();
  ///
  void lowercase();
 
  static int
  compare_nocase( string const &, string const & );

  ///
  static int
  find_nocase(vector<string> const &, string const &pattern);

  /**
     Removes leading 'c's from 's'.
   */
  void strip_front( Char c = ' ' );

  /**
     Removes trailing 'c's from 's'.
   */
  void
  strip_back( Char c = ' ' );

  /**
     Removes leading and trailing 'c's from 's'.
   */
  void
  strip( Char = ' ' );

  // splits s into whitespace separated words
  int
  split( vector<string> &w );

  /**
     Reads Chars from 'stream' into string 's' until marker
       or EOF is encountered.
     Throws away 'marker'.
     Clears 's' before reading.
     Returns:
       1 if EOF and s.length() == 0 (i.e. if done)
       0 otherwise
  */
  int read( istream &stream, Char marker = '\n');

  ///
  int to_int( int & ) const;
  ///
  int to_int( int unsigned & ) const;
  ///
  int to_int( int short & ) const;
  ///
  int to_int( int unsigned short & ) const;
  ///
  int to_int( int long & ) const;
  ///
  int to_int( int unsigned long & ) const;
  ///
  int to_real( float & ) const;
  ///
  int to_real( double & ) const;
  ///
  int to_real( double long & ) const;

  ///
  void to_string( int x );
  ///
  void to_string( int unsigned x );
  ///
  void to_string( int short x );
  ///
  void to_string( int unsigned short x );
  ///
  void to_string( int long x );
  ///
  void to_string( int unsigned long x );
  ///
  void to_string( float x );
  ///
  void to_string( double x );
  ///
  void to_string( double long x );

  ///
  void to_string( complex<float> const &x );
  ///
  void to_string( complex<double> const &x );
  ///
  void to_string( complex<double long> const &x );
  
  /**
     read a word, eating initial whitespace, up to whitespace or eof
  */
  int
  read_word( istream & );

  /**
     read a word, eating initial whitespace, up to whitespace or eof
     adjust to line_number and column_number
  */
  int
  read_word( istream& in, unsigned &line_number,
             unsigned &column_number );

protected:
  /// returns copy of *this
  Char *allocate() const;
  ///
  static void deallocate( Char * );
};

} // namespace base

#if defined(__sgi) && !defined(__GNUG__)
#pragma reset woff 1375
#endif

#endif // !stringx_INCLUDED
