// Copyright (C) 2000 Nicholas Schmitt nick@math.umass.edu

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <new.h>
#if defined(__sgi)
#include <exception.h>
#endif
#include "System.H"

namespace base {

static SystemStruct systemstruct;
SystemStruct * const System = &systemstruct;

SystemStruct::
SystemStruct()
{
  name("");
  file(__FILE__,__LINE__);
  set_new_handler(newhandler);
  set_terminate(terminate);
}

void
SystemStruct::
warn( const char *fmt, ...)
{
  fprintf( stderr, "%s: warning: ", _name );

  va_list ap;
  va_start(ap, fmt);
  vfprintf(stderr, fmt, ap);
  va_end(ap);

  fprintf( stderr, " (%s:%d)\n", _file, _line );
}

void
SystemStruct::
warn( bool flag )
{
  if (!flag) { return; }

  fprintf( stderr, "%s: warning", _name );
  fprintf( stderr, " (%s:%d)\n", _file, _line );
}

void
SystemStruct::
warn( bool flag, const char *fmt, ...)
{
  if (!flag) { return; }

  fprintf( stderr, "%s: warning: ", _name );

  va_list ap;
  va_start(ap, fmt);
  vfprintf(stderr, fmt, ap);
  va_end(ap);

  fprintf( stderr, " (%s:%d)\n", _file, _line );
}

void
SystemStruct::
die( bool flag )
{
  if (!flag) { return; }

  fprintf( stderr, "%s: error", _name );
  fprintf( stderr, " (%s:%d)\n", _file, _line );
  abort();
}

void
SystemStruct::
die( const char *fmt, ...)
{
  fprintf( stderr, "%s: error: ", _name );

  va_list ap;
  va_start(ap, fmt);
  vfprintf(stderr, fmt, ap);
  va_end(ap);

  fprintf( stderr, " (%s:%d)\n", _file, _line );

  abort();
}

void
SystemStruct::
die( bool flag, const char *fmt, ...)
{
  if (!flag) { return; }

  fprintf( stderr, "%s: error: ", _name );

  va_list ap;
  va_start(ap, fmt);
  vfprintf(stderr, fmt, ap);
  va_end(ap);

  fprintf( stderr, " (%s:%d)\n", _file, _line );

  abort();
}

void
SystemStruct::
trace()
{
  fprintf( stderr, "%s: trace", _name );
  fprintf( stderr, " (%s:%d)\n", _file, _line );
}

void
SystemStruct::
newhandler()
{
  System->file(__FILE__,__LINE__);
  System->die( "out of memory" );
}

void
SystemStruct::
terminate()
{
  System->file(__FILE__,__LINE__);
  System->die( "top-level exception" );
}

} // namespace base
