// Copyright (C) 2000 Nicholas Schmitt nick@math.umass.edu

#ifndef System_INCLUDED
#define System_INCLUDED

#include "Use.h"

namespace base {

struct SystemStruct
{
  SystemStruct();
  void warn( bool flag=true );
  void warn( char const *format, ... );
  void warn( bool flag, char const *format, ... );
  void die( bool flag=true );
  void die( char const *format, ... );
  void die( bool flag, char const *format, ... );
  void trace();
  char const *name() { return _name; }
  void name( char const *name ) { _name = name; }
  void file( char const *file, int line )
    { _file = file; _line = line; }
private:
  char const *_name;
  char const *_file;
  int _line;
  static void newhandler();
  static void terminate();
};

extern SystemStruct * const System;

// macros are used here to automatically insert file/line info
// WARN and DIE use macros in a non-standard way to allow
// variable numbers of arguments
#undef WARN
#undef WARN_IF
#undef DIE
#undef DIE_IF
#undef TRACE

#define WARN \
 System->file(__FILE__,__LINE__); System->warn
#define WARN_IF WARN
#define DIE \
 System->file(__FILE__,__LINE__); System->die
#define DIE_IF DIE
#define TRACE \
 System->file(__FILE__,__LINE__); System->trace();

} // namespace base

#endif // !System_INCLUDED
