// Copyright (C) 2000 Nicholas Schmitt nick@math.umass.edu

#ifndef timer_INCLUDED
#define timer_INCLUDED

#include <unistd.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/times.h>
#ifdef __GNUG__
#include <time.h>
#endif
#include "String.H"

namespace base {

/**
  Get user/system/elapsed time.

  usage:

  #include "timer.H"
  
  string user_time, system_time, elapsed_time;
  Timer timer;

  // start stopwatch (optional: the constructor does this)
  timer.reset();

  ...
  
  // report the time
  timer.read( user_time, system_time, elapsed_time );  
  cerr << "time=" << user_time << "," << system_time << ","
       << elapsed_time << endl;

  file timer.H
  */
class Timer
{
public:

  /// constructor (initializes timer)
  inline Timer();

  /// reset (initializes timer)
  inline void reset();

  /// return user/system/elapsed time in 1/100'ths of seconds
  void read( long &, long &, long & ) const;

  /// return user/system/elapsed time in seconds
  void read( float &, float &, float & ) const;

  /**
     return user/system/elapsed time in seconds as strings
     (two-digit precision)
  */
  void read( string &, string &, string & ) const;
  
private:
  ///
  tms system_time0;
  ///
  clock_t elapsed_time0;
};


Timer::
Timer()
{
  reset();
}


void
Timer::
reset()
{
  // initialize system_time0 and elapsed_time0
  elapsed_time0 = times( &system_time0 );
}

} // namespace base

#endif // !timer_INCLUDED
