// Copyright (C) 2000 GANG http://www.gang.umass.edu

#include <ctype.h>
#include <iostream.h>
#include <iomanip.h>
#include <GL/glut.h>
#include <algo.h>
#include "MathReal.H"
#include "curve.H"
#include "Curve.H"

namespace curve {

Curve *Curve::_me = (Curve *)NULL;

/*
  static members
*/
Curve::Color const Curve::_background_color[2] = {
  Color( 0.0f, 0.0f, 0.0f ),
  Color( 1.0f, 1.0f, 1.0f ) };

Curve::Color const Curve::_first_mousepoint_color[2] = {
  Color( 1.0f, 0.5f, 0.5f ),
  Color(0.0, 0.0, 0.0) };

Curve::Color const Curve::_mousepoint_color[2] = {
  Color( 1.0f, 1.0f, 1.0f ),
  Color( 1.0f, 1.0f, 1.0f ) };

Curve::Color const Curve::_point_color[2] = {
  Color( 0.0f, 0.0f, 1.0f ),
  Color( 0.0f, 0.0f, 1.0f ) };

Curve::Color const Curve::_point_color2[2] = {
  Color( 1.0f, 1.0f, 1.0f ),
  Color( 0.0f, 0.0f, 0.0f ) };

Curve::Color const Curve::_line_color[2] = {
  Color( 0.6f, 0.6f, 0.6f ),
  Color( 0.5f, 0.5f, 0.5f ) };

Curve::Color const Curve::_label_color[2] = {
  Color( 1.0f, 1.0f, 0.0f ),
  Color( 0.5f, 0.5f, 0.0f ) };

Curve::Color const Curve::_label_color2[2] = {
  Color( 0.0f, 0.0f, 1.0f ),
  Color( 0.0f, 0.0f, 1.0f ) };

float Curve::_first_mousepoint_size = 4.0f;
float Curve::_mousepoint_size = 0.0f;
float Curve::_point_size = 4.0f;
float Curve::_line_size = 1.0f;


/*
  -------------------------------------------
  constuctors, destructors, assignment operators
*/
Curve::
Curve(
  int width, int height,
  int argc, char *argv[] )
    :
    Plane( width, height, argc, argv ),
    _polygon((Polygon *)NULL),
    _edge((EdgeX *)NULL),
    _face((FaceX *)NULL),
    _graph((GraphVector *)NULL)
{
  _iteration = 0;
  _draw_mousepoints = false;

  // initialize the graphical window
  // glEnable(GL_POINT_SMOOTH);
  // glEnable(GL_BLEND);

  _labels = 0;
  _color = 0;
  _me = this;

  _menu = glutCreateMenu( menu_cb_ );
  glutAddMenuEntry( "[g]raph", MENU_GRAPH );
  glutAddMenuEntry( "[l]abels", MENU_LABELS );
  glutAddMenuEntry( "re[o]rient", MENU_ORIENT );
  glutAddMenuEntry( "degree [+]1", MENU_ADD );
  glutAddMenuEntry( "degree [-]1", MENU_SUB );
  glutAddMenuEntry( "[c]olor", MENU_COLOR );
  glutAddMenuEntry( "[q]uit", MENU_EXIT );
  glutAttachMenu(GLUT_RIGHT_BUTTON);
}

Curve::
Curve( Curve const &a )
    :
    Plane(a)
{
  throw ERR_INTERNAL;
}

Curve::
~Curve()
{
}

Curve &
Curve::
operator=( Curve const & )
{
  throw ERR_INTERNAL;
  //return *this;
}


/*
  -------------------------------------------
  geometry computations
*/

void
Curve::
clear()
{
  _mousepoint.clear();
  clear_window();
}

void
Curve::
compute()
{
  // start timer
  _timer.reset();

  if ( _polygon != (Polygon *)NULL ) { delete _polygon; }
  _polygon = (Polygon *)NULL;

  if ( _edge != (EdgeX *)NULL ) { delete _edge; }
  _edge = (EdgeX *)NULL;

  if ( _face != (FaceX *)NULL ) { delete _face; }
  _face = (FaceX *)NULL;

  if ( _graph != (GraphVector *)NULL ) { delete _graph; }
  _graph = (GraphVector *)NULL;


  /*
    Call the geometry sub-functions
    This function should be called when the user has completed drawing
    the diagram with the mouse
  */

  try
  {
    _polygon = new Polygon(_mousepoint);
  }
  catch (Exception)
  {
    /*std*/::cerr << "*** Could not parse mouse points.\n";
    return;
  }

  try
  {
    _edge = new EdgeX(*_polygon);
  }
  catch (Exception e )
  {
    if ( e == ERR_FACES )
    {
      /*std*/::cerr << "*** Could not find faces.\n";
      return;
    }
    else
    {
      /*std*/::cerr << "*** Could not parse curve.\n";
      return;
    }
  }
#if 0
  _edge->dump(/*std*/::cerr);
#endif
  // draw the intersection points on the graphical window
  draw_intersection_points();

  if (_labels == 1)
  {
    label_faces();
    label_edges();
  }

  try
  {
    _face = new FaceX( *_edge );
  }
  catch (Exception const & )
  {
    /*std*/::cerr << "*** Could not compute degrees.\n";
    return;
  }

  if (_labels == 0)
  {
    label_degrees();
  }

#if 0
  _face->dump( /*std*/::cerr );
#endif

#if 0
  dump(/*std*/::cerr);
#endif

  compute_graph();

  // report the results
  report( /*std*/::cout );
}


void
Curve::
compute_graph()
{
  if ( _face->has_degrees() )
  {
  try
  {
    _graph = new GraphVector( *_face );
  }
  catch (Exception const & )
  {
    /*std*/::cerr << "*** Could not compute graphs.\n";
    return;
  }
  }
}

void
Curve::
report( /*std*/::ostream &o )
{
  /*
    Report the computed data in two channels:
      - some data is send to the terminal as text
      - other data is shown on the R^2 window as color or text
  */
  o << "Turning number:           "
    << _polygon->turning_number() << "\n";

  o << "Has non-negative degrees: "
    << (_face->has_degrees() ? "yes" : "no") << "\n";

  o << "Euler characteristic:     " << _face->euler_characteristic() << "\n";

  uint n = 0;
  if (_face->size() == 0)
  {
    n = 1;
  }
  else if ( _graph != (GraphVector *)NULL)
  {
    n = _graph->size();
  }
  o << "Number of graphs:         " << n << "\n";


#if 0
  if (_vertex_count == 0 )
  {
    o << "Bounds: yes\n";
  }
  else if ( abs(_turning_number) != 1 )
  {
    o << "Bounds: no\n";
  }
  else if ( !_has_degrees )
  {
    o << "Bounds: no\n";
  }
  else
  {
    o << "Bounds: unknown\n";
  }
#endif

#if 0
  o << "  ";
  if (_vertex_count == 0 )
  {
    o << "(This curve is an embedded disk.)\n";
  }
  else if ( abs(_turning_number) != 1 )
  {
    o << "(This curve does not bound an immersed disk since its turning number is " << _turning_number << ".)\n";
  }
  else if ( _degree.size() ==0 )
  {
    o << "(This curve does not bound an immersed disk since degrees cannot be assigned.)\n";
  }
  else
  {
    o << "(This curve is a candidate for bounding an immersed disk.)\n";
  }
#endif

  // timer

  string user_time, system_time, elapsed_time;
  _timer.read( user_time, system_time, elapsed_time );  

  o << "Computation time:         " << user_time << "s\n";

  o << /*std*/::flush;
}

void
Curve::
draw_intersection_points()
{
  /*
    Draws the points where the curve intersects itself
    onto the graphical window
  */

  if (_edge == (EdgeX *)NULL ) { return; }

  // draw large white square
  set_point_size(_point_size+2);
  set_color( _point_color2[_color] );
  draw_points( _edge->intersection_point() );

  // draw small blue square on top
  set_point_size(_point_size);
  set_color( _point_color[_color] );
  draw_points( _edge->intersection_point() );

  flush();
}

void
Curve::
label_edges()
{
  /*
    Label the edges (segements of curve between its intersection points).
    These are written slightly along the oriented curve from the intersection
    point.
  */

  float const SCALE = 30.0;

  set_color( _label_color2[_color] );

  /*std*/::vector<EdgeElement>::const_iterator i;
  for ( i = _edge->edge().begin(); i != _edge->edge().end(); i++ )
  {
    uint a0 = (*i).polygon();
    Point p0 = (*_polygon)[a0];
    uint a1 = next_(a0, *_polygon);
    Point p1 = (*_polygon)[a1];
    Point v = p1 - p0;
    v *= SCALE / v.abs();

    Point p = _edge->intersection_point()[(*i).intersection_point()];

    write_number( (int)(i - &(*_edge)[0]), p + v );
  }
}

void
Curve::
label_faces()
{
  /*
    Put numerical labels on the faces in the graphical window.
    The numbers are placed 4 around each intersection point of the curve.
  */

  float const SCALE = 10.0;

  set_color( _label_color[_color] );

  /*std*/::vector<EdgeElement>::const_iterator i;
  for ( i = _edge->edge().begin(); i != _edge->edge().end(); i++ )
  {
    uint a0 = (*i).polygon();
    Point p0 = (*_polygon)[a0];
    uint a1 = next_(a0, *_polygon);
    Point p1 = (*_polygon)[a1];
    Point v = p1 - p0;
    v *= SCALE / v.abs();
    Point w = v.perp();

    Point p = _edge->intersection_point()[(*i).intersection_point()];

    if ((*i).orient() == 0)
    {
      /*std*/::vector<EdgeElement>::const_iterator j =
        &_edge->edge()[ prev_(i - &(*_edge)[0], *_edge) ];
      write_number( (int)(*i).face(0), p + v + w );
      write_number( (int)(*i).face(1), p + v - w );
      write_number( (int)(*j).face(0), p - v + w );
      write_number( (int)(*j).face(1), p - v - w );
    }
  }
}


void
Curve::
label_degrees()
{
  if ( _edge == (EdgeX *)NULL ) { return; }

  set_color( _label_color[_color] );

  float const SCALE = 10.0;

  /*std*/::vector<EdgeElement>::const_iterator i;
  for ( i = _edge->edge().begin(); i != _edge->edge().end(); i++ )
  {
    uint a0 = (*i).polygon();
    Point p0 = (*_polygon)[a0];
    uint a1 = next_(a0, *_polygon);
    Point p1 = (*_polygon)[a1];
    Point v = p1 - p0;
    v *= SCALE / v.abs();
    Point w = v.perp();

    Point p = _edge->intersection_point()[(*i).intersection_point()];

    if ((*i).orient() == 0)
    {
      /*std*/::vector<EdgeElement>::const_iterator j =
        &(*_edge)[prev_(i - &(*_edge)[0], *_edge)];
      write_number( (*_face)[(*i).face(0)].degree, p + v + w );
      write_number( (*_face)[(*i).face(1)].degree, p + v - w );
      write_number( (*_face)[(*j).face(0)].degree, p - v + w );
      write_number( (*_face)[(*j).face(1)].degree, p - v - w );
    }
  }
}

/*
  -------------------------------------------
  EVENT CALLBACKS
  These are called when the user presses a key or presses/moves the mouse
  The name of each of these functions ends in '_cb'
 */

void
Curve::
reshape_cb( int width, int height )
{
  /*
    This method is called when the user resizes the window.
  
    Rescale _point so that it keeps its aspect ratio but fits
    in the window

    ** TO DO: Has northwest gravity; centering the image would be better.
  */

  if ( _polygon == (Polygon *)NULL || _polygon->empty() ) { return; }
  if ( width == 0 || height == 0 ) { return; }
  if (_image_width == 0 || _image_height == 0 ) { return; }
  
  float s = MathReal::min(
    float(width)/float(_image_width),
    float(height)/float(_image_height) );

  _image_width = (int)rint(_image_width * s);
  _image_height = (int)rint(_image_height * s);
  
  _polygon->rescale(s);
  _edge->rescale(s);
}


void
Polygon::
rescale( real s )
{
  if ( s != 1.0 )
  {
    /*std*/::vector<Point>::iterator i;
    for ( i = _polygon.begin(); i != _polygon.end(); i++ )
    {
      (*i) *= s;
    }
  }
}


void
EdgeX::
rescale( real s )
{
  if ( s != 1.0 )
  {
    /*std*/::vector<Point>::iterator i;
    for ( i = _intersection_point.begin();
          i != _intersection_point.end(); i++ )
    {
      (*i) *= s;
    }
  }
}

void
Curve::
display_cb()
{
  /*
    This method is called when the window gets exposed and has to be redrawn.
  */

  // clear the display
  clear_window();

  // draw the curve
  set_color( _line_color[_color] );
  set_line_size( _line_size );

  draw_lines( _mousepoint, true );

  // draw the mouse points

  if (_draw_mousepoints)
  {
    set_point_size( _mousepoint_size );
    set_color( _mousepoint_color[_color] );
    draw_points( _mousepoint );

    if (!_mousepoint.empty())
    {
      set_point_size( _first_mousepoint_size);
      set_color( _first_mousepoint_color[_color] );
      draw_point( _mousepoint.front() );
    }
  }

  // draw the intersection points
  draw_intersection_points();

  if (_labels == 0)
  {
    label_degrees();
  }
  else if (_labels == 1)
  {
    label_faces();
    label_edges();
  }

  flush();
}


void
Curve::
mouse_cb( int button, int state, int x, int y)
{
  /*
    This method is called when the user presses or releases a mouse button.

    *** TO DO: Handle the situation in which the user clicks but doesn't
    drag the mouse
  */
  USE(button);

  if (state == 0) {
    // mouse press event

    clear();
    /*std*/::cout << "-----------------------------------------\n";
    /*std*/::cout.flush();
    _iteration++;

    Point p;
    p.x = x;
    p.y = y;
    _mousepoint.push_back(p);

    if (_draw_mousepoints)
    {
      set_color( _first_mousepoint_color[_color] );
      set_point_size( _first_mousepoint_size );
      draw_point( p );
      flush();
    }
  }
  else if (state==1)
  {
    // mouse release event
    if (_mousepoint.size() >= 2 )
    {
      // join beginning to end
      if (_mousepoint.size() >= 2 )
      {
        set_color( _line_color[_color] );
        draw_line( _mousepoint.back(), _mousepoint.front() );
      }
    }
    flush();
    compute();

    _image_width = (int)rint(_width);
    _image_height = (int)rint(_height);
  }
}

void
Curve::
motion_cb( int x, int y)
{
  /*
    This method is called when the user drags the mouse.
  */

  Point p;
  p.x = real(x);
  p.y = real(y);
  set_color( _line_color[_color] );

  // plot line
  uint n = _mousepoint.size();
  if ( n >= 1 )
  {
    if ( p != _mousepoint[n-1] )
    {
      set_color( _line_color[_color] );
      set_line_size( _line_size );
      draw_line( _mousepoint[n-1], p );

      if (_draw_mousepoints) {
        // plot point
        set_point_size( _mousepoint_size );
        set_color( _mousepoint_color[_color] );
        draw_point( p );
      }

      _mousepoint.push_back(p);
      flush();
    }
  }
}

void
Curve::
key_cb( unsigned char key, int x, int y )
{
  /*
    This method is called when the user presses a key.
  */
  USE(x);
  USE(y);

  char unsigned k = tolower(key);

  if ( k == 'c' )
  {
    menu_cb(MENU_COLOR);
  }
  else if( k == 'g' )
  {
    menu_cb(MENU_GRAPH);
  }
  else if( k == 'l' )
  {
    menu_cb(MENU_LABELS);
  }
  else if( k == '+' )
  {
    menu_cb(MENU_ADD);
  }
  else if( k == '-' )
  {
    menu_cb(MENU_SUB);
  }
  else if( k == 'o' )
  {
    menu_cb(MENU_SUB);
  }
  else if( k == 'q' )
  {
    menu_cb(MENU_EXIT);
  }
}

void
Curve::
menu_cb( int mode )
{
  if ( mode ==  MENU_COLOR )
  {
    _color = 1 - _color;
    set_background_color( _background_color[_color] );
  }
  else if ( mode == MENU_GRAPH )
  {
    _face->dump( /*std*/::cout );
    _face->dump_degrees( /*std*/::cout );
    _graph->dump( /*std*/::cout );
  }
  else if ( mode == MENU_LABELS )
  {
    _labels = 1 - _labels;
    display_cb();
  }
  else if ( mode == MENU_ADD )
  {
    _face->add_degree(1);
    display_cb();
    compute_graph();
    /*std*/::cout << "\n";
    report( /*std*/::cout );
  }
  else if ( mode == MENU_SUB )
  {
    _face->add_degree(-1);
    display_cb();
    compute_graph();
    /*std*/::cout << "\n";
    report( /*std*/::cout );
  }
  else if ( mode == MENU_ORIENT )
  {
    // *** TO DO: fix this hack
    /*std*/::cout << "\n";
    reverse(_mousepoint.begin(), _mousepoint.end());
    compute();
    display_cb();
  }
  else if ( mode == MENU_EXIT )
  {
    // There is no way to return from glutMainLoop() directly; the
    // C++ exception mechanism is used instead.
    glutDestroyMenu(_menu);
    glutDestroyWindow(_window);
#if defined(__sgi)
    throw (int)0;
#else
    exit(0);
#endif
  }
}

void
Curve::
menu_cb_( int mode )
{
  _me->menu_cb(mode);
}

} // namespace curve
