// Copyright (C) 2000 GANG http://www.gang.umass.edu
#ifndef Curve_INCLUDED
#define Curve_INCLUDED

#include <iostream.h>
#include "Timer.H"
#include "Polygon.H"
#include "Edge.H"
#include "Face.H"
#include "Graph.H"
#include "Plane.H"
#include "curve.H"

namespace curve {

/*
  ----------------------------------------------------------------
  Curve
  ----------------------------------------------------------------
*/


class Curve
  :
  public Plane
{

public:

  Curve(
    int width, int height,
    int argc, char *argv[] );

  virtual ~Curve();

protected:

  static const Color _background_color[2];
  static const Color _first_mousepoint_color[2];
  static const Color _mousepoint_color[2];
  static const Color _point_color[2];
  static const Color _point_color2[2];
  static const Color _line_color[2];
  static const Color _label_color[2];
  static const Color _label_color2[2];

  static float _first_mousepoint_size;
  static float _mousepoint_size;
  static float _point_size;
  static float _line_size;

  void clear();
  void compute();
  void compute_graph();

  void report( /*std*/::ostream &o );
  void dump( /*std*/::ostream & );
  void draw_intersection_points();

  void label_edges();
  void label_faces();
  void label_degrees();

  // event handlers
  virtual void reshape_cb( int width, int height );
  virtual void key_cb(unsigned char key, int x, int y);
  virtual void mouse_cb( int button, int state, int x, int y);
  virtual void display_cb();
  virtual void motion_cb( int x, int y );

  // DATA

  bool _draw_mousepoints;
  /*std*/::vector<Point> _mousepoint;

  int _image_width, _image_height;

  Polygon *_polygon;
  EdgeX *_edge;
  FaceX *_face;
  GraphVector *_graph;

  uint _iteration;
  Timer _timer;

private:

  // disabled
  Curve( Curve const & );
  Curve &operator=( Curve const & );

  enum { MENU_COLOR, MENU_GRAPH, MENU_LABELS,
         MENU_ADD, MENU_SUB, MENU_ORIENT, MENU_EXIT };
  void menu_cb( int mode );
  static void menu_cb_( int mode );
  int _color;
  static Curve *_me;
  int _labels;

  int _menu;
};

} // namespace curve

#endif
