// Copyright (C) 2000 GANG http://www.gang.umass.edu
#ifndef Edge_INCLUDED
#define Edge_INCLUDED

#include <iostream.h>
#include <vector.h>
#include <map.h>
#include "Polygon.H"
#include "curve.H"

namespace curve {

/*
  ----------------------------------------------------------------
  Edge
  ----------------------------------------------------------------
*/


class EdgeElement
{

/*

   An 'EdgeElement' is a struct as follows:
  struct EdgeElement {
    uint _cross;   // the index of the edge that crosses this edge
    uint _face[2]; // face[0]=left face; face[1]=right face
    uint _point;   // index into _intersection_point
    uint _p;       // index into polygon of the point just before this
    bool3 _orient;       // orientation (see below) 
  };

   orientation = 0   orientation = 1
             ^                    |
             |                    |
     this -------->       this -------->
             |                    |
             |                    v
*/

public:

  EdgeElement();
  EdgeElement(
    EdgeElement *prev, EdgeElement *next, EdgeElement *cross, bool3 orient,
    uint intersection_point, uint polygon );

  inline EdgeElement const *prev() const { return _prev; }
  inline EdgeElement *prev() { return _prev; }

  inline EdgeElement const *next() const { return _next; }
  inline EdgeElement *next() { return _next; }
#if 0
  inline EdgeElement const *prev_left() const
    { return orient() == LEFT ? cross() : cross()->prev(); }
  inline EdgeElement *prev_left()
    { return orient() == LEFT ? cross() : cross()->prev(); }

  inline EdgeElement const *prev_right() const
    { return orient() == LEFT ? cross()->prev() : cross(); }
  inline EdgeElement *prev_right()
    { return orient() == LEFT ? cross()->prev() : cross(); }

  inline EdgeElement const *next_left() const
    { return _next->prev_left(); }
  inline EdgeElement *next_left()
    { return _next->prev_left(); }

  inline EdgeElement const *next_right() const
    { return _next->prev_right(); }
  inline EdgeElement *next_right()
    { return _next->prev_right(); }
#endif
  inline EdgeElement const *cross() const { return _cross; }
  inline EdgeElement *cross() { return _cross; }

  inline uint face(uint i) const { return _face[i]; }
  inline uint &face(uint i) { return _face[i]; }

  enum Orient {LEFT = 0, RIGHT = 1, NOTHING = -1};
  inline Orient orient() const { return _orient; }
  inline Orient &orient() { return _orient; }

  uint polygon() const { return _polygon; }
  uint &polygon() { return _polygon; }

  uint intersection_point() const { return _intersection_point; }
  uint &intersection_point() { return _intersection_point; }

  static void follow_face( EdgeElement const * &e, int &direction );

public:
  EdgeElement *_prev;
  EdgeElement *_next;
  EdgeElement *_cross;

  uint _face[2];
  uint _intersection_point;
  uint _polygon;

  Orient _orient;
  /*std*/::vector<uint> connect[2];
};


class EdgeX
{
public:
  EdgeX( Polygon const &polygon)  throw ( Exception ) :
      _polygon(polygon),
      _edge(),
      _intersection_point(),
      _outside_face(UNDEFINED_UINT)
    { compute(); }

  typedef /*std*/::vector</*std*/::pair<uint, uint> > VertexList;
  typedef /*std*/::map< uint, /*std*/::pair<uint,uint> > VertexHash;

  void clear() { _edge.clear(); }
  uint size() const { return _edge.size(); }
  EdgeElement const &operator[](uint n) const { return _edge[n]; }

  /*std*/::vector<EdgeElement> const &edge() const { return _edge; }

  /*std*/::vector<Point> const &intersection_point() const
    { return _intersection_point; }

  void rescale( real );

  uint face_count() const { return _face_count; }

  uint outside_face() const { return _outside_face; }

  void dump( /*std*/::ostream & );

protected:
  void compute() throw ( Exception );

  void compute_intersection_points() throw ( Exception );

  void compute_intersection_points(
    VertexList const & );

  void make_edges(    
    VertexList const &vertex_list,
    VertexHash const &vertex_hash);

  void compute_orientations();
  void compute_faces() throw ( Exception );
  void compute_outside_face() throw ( Exception );

private:
  Polygon const &_polygon;

  // the edges
  /*std*/::vector<EdgeElement> _edge;

  // /*std*/::vector of points in R^2 created by mouse
  /*std*/::vector<Point> _intersection_point;

  uint _face_count;
  uint _outside_face;


  // disabled
  EdgeX &operator=( EdgeX const & ) { return *this; }

};

} // namespace curve

#endif
