// Copyright (C) 2000 GANG http://www.gang.umass.edu
#ifndef Face_INCLUDED
#define Face_INCLUDED

#include <iostream.h>
#include <vector.h>
#include "curve.H"
#include "Edge.H"

namespace curve {

/*
  ----------------------------------------------------------------
  Face
  ----------------------------------------------------------------
*/

struct FaceElement
{
  /*
    struct FaceElement:

    f.face[i] is the face adjacent to f across edge f.edge[i]

    f.dot[i] is a vector of f.edge.size() uints

    f.dot[i][j] = n means that dot i of this face is connected across
      edge j to dot n in the opposite face.
  */

  struct EdgeOfFace {
    uint edge;
    uint opp_face;
    uint opp_edge;
  };

  int degree;

  /*std*/::vector<EdgeOfFace> edge;
};


class FaceX
{
public:
  FaceX( EdgeX const &edge ) :
      _face(),
      _edge(edge),
      _has_degrees( UNDEFINED_BOOL ),
      _max_degree( UNDEFINED_INT )
    { compute(); }

  void clear() { _face.clear(); }
  uint size() const { return _face.size(); }
  bool empty() const { return _face.empty(); }
  FaceElement const &operator[]( uint n ) const { return _face[n]; }
  /*std*/::vector<FaceElement> const &face() const { return _face; }
  EdgeX const &edge() const { return _edge; }
  bool has_degrees() const;
  int max_degree() const { return _max_degree; }
  int euler_characteristic() const { return _euler_characteristic; }
  void dump( /*std*/::ostream & ) const;
  void dump_degrees( /*std*/::ostream &o ) const;

  void add_degree( int );
protected:
  void compute() throw ( Exception );
  void construct();
  void compute_degrees() throw ( Exception );
  void compute_faces();
  void compute_euler_characteristic();
  void compute_max_degree();
  void compute_has_degrees();
private:
  /*std*/::vector<FaceElement> _face;
  EdgeX const &_edge;
  bool3 _has_degrees;
  int _max_degree;
  int _euler_characteristic;

  // disabled
  FaceX &operator=( FaceX const & ) { return *this; }
};

} // namespace curve

#endif
