// Copyright (C) 2000 GANG http://www.gang.umass.edu
#ifndef Graph_INCLUDED
#define Graph_INCLUDED

#include <vector.h>
#include "Face.H"
#include "curve.H"

namespace curve {

/*
  ----------------------------------------------------------------
  Graph
  ----------------------------------------------------------------
*/

class Graph
{
public:
  Graph( FaceX const &face ) :
      _graph(),
      _face(face)
    {}

  bool same( Graph const & ) const;

  void dump( /*std*/::ostream & ) const;


  Graph &operator=( Graph const &a )
    { _graph = a._graph; return *this; }

protected:

  struct FaceDotEdge {
    uint face;
    uint dot;
    uint edge;
    FaceDotEdge() :
        face(UNDEFINED_UINT), dot(UNDEFINED_UINT), edge(UNDEFINED_UINT) {}
    FaceDotEdge( uint face_, uint dot_, uint edge_ ) :
        face(face_), dot(dot_), edge(edge_) {}
  };
  
  friend /*std*/::ostream &
  operator<<( /*std::*/::ostream &o, FaceDotEdge const &x );

  bool diamond_check() const;
  int diamond_check_edge1( FaceDotEdge const &x ) const;

  bool diamond_check( FaceDotEdge const &x, FaceDotEdge const &y ) const;
  bool diamond_check_edge( FaceDotEdge const &x, FaceDotEdge const &y ) const;

  bool
  next_connection(
    FaceDotEdge &y ) const;

  void
  choose_connection(
    FaceDotEdge &x, FaceDotEdge &y,
    /*std*/::vector</*std*/::pair<uint,uint> > const &
    ) const;

  void insert_edge( FaceDotEdge const &x, FaceDotEdge const &y );
  void remove_edge( FaceDotEdge const &x, FaceDotEdge const &y );

  struct DictElement
  {
    bool3 done;
    /*std*/::vector<uint> dot;
  };

  bool3
  dictionary_entry(
    Graph const &graph0,
    uint face,
    uint edge,
    /*std*/::vector<DictElement> &dict ) const;

  bool3
  dictionary_entry2(
    Graph const &graph0,
    uint face,
    uint edge,
    /*std*/::vector<DictElement> &dict ) const;

  static void space( /*std*/::ostream &, uint n );

private:
  typedef /*std*/::vector<uint> Dot;
  typedef /*std*/::vector<Dot> FaceDot;

  /*std*/::vector<FaceDot> _graph;
  vector<vector<uint> > _connection_count;
  FaceX const &_face;

  void around( FaceDotEdge &b, FaceDotEdge const &a, bool ccw ) const;

  friend class GraphVector;
};


class GraphVector
{
  // collection of connection graphs
public:
  GraphVector( FaceX const &face ) :
    _graph(),
    _face(face)
//    _edge(face._edge)
    { compute(); }

  uint size() const { return _graph.size(); }
  Graph const &operator[]( uint n ) const { return _graph[n]; }

  void dump( /*std*/::ostream & );

protected:
  void compute() throw ( Exception );
  void construct( Graph & );
  void uniquify();
  void recurse( Graph &, uint level );

private:

  /*std*/::vector<Graph> _graph;
  FaceX const &_face;

  static FaceX const *face_list_face;

  static bool face_list_compare(
    /*std*/::pair<uint, uint> const &, /*std*/::pair<uint, uint> const & );

  /*std*/::vector</*std*/::pair<uint, uint> > _sorted_face_list;

  // disabled
  GraphVector &operator=( GraphVector const & ) { return *this; }
};

} // namespace curve

#endif
