// Copyright (C) 2000 GANG http://www.gang.umass.edu

/*
  -------------------------------------------
  Analyses a closed curve in R^2. Computes the turning number of the curve
  and whether the curve bounds an immersed disk.

  ALGORITHM:

  1. The user draws a curve on an graphical window representing R^2. This
  is represented as a sequence (loop) of points in R^2.

  2. The turning number of the curve is computed by considering it as
  a polygon and adding the changes in angle at the corners. Any closed
  curve which bounds an immersed disk has turning number +1 or -1.

  3. From this sequence, the intersection points are detected, together
  with information expressing the topology and geometry (connectivity
  and orientation) of these points, the edges connecting them and the
  faces these edges bound.

  4. Assuming there is an immersion f, compute the degree of f in each
  face. If this cannot be done consistently, the curve does not bound
  an immersed disk.

  5. Compute the "connection graph" for the curve. The connection graph
  is graph that is dual to the cell-decomposition of the preimage manifold.

  -------------------------------------------
*/


#ifndef Immersion_INCLUDED
#define Immersion_INCLUDED

namespace curve {

class Curve;

class Immersion
{

public:
  Immersion(
    int width, int height,
    int argc, char *argv[] );
  virtual ~Immersion();

  void loop();
protected:

private:
  Curve *_curve;

  // disabled
  Immersion( Immersion const & );
  Immersion &operator=( Immersion const & );
};

} // namespace curve

#endif // !Immersion_INCLUDED
