// Copyright (C) 2000 GANG www.gang.umass.edu

#include <stdlib.h>
#include <math.h>
#include <iostream.h>
#include <GL/glut.h>
#include "System.H"
#include "String.H"
#include "Plane.H"

namespace curve {

Plane *Plane::_this = (Plane *)NULL;


Plane::
Plane(
  int width, int height,
  int argc, char *argv[] )
{
  init(width, height, argc, argv);
}

Plane::
Plane( Plane const & )
{
}

Plane::
~Plane()
{
}

Plane &
Plane::
operator=( Plane const & )
{
  return *this;
}

void
Plane::
init(
  int width, int height,
  int argc, char *argv[] )
{
  // Set static pointer "_this" to this.
  // This is necessary to make the glut callbacks work
  _this = this;

  // cache the window size
  //  _width = width;
  //  _height = height;

  int win;
  USE(win);

  // initialize GLUT system
  glutInit(&argc, argv);

  glutInitDisplayMode(GLUT_RGB);

  glutInitWindowSize(width, height);

  // create window
  _window = glutCreateWindow("Does this curve bound an immersed surface?");

  // set background to black
  glClearColor(0.0,0.0,0.0,0.0);

  // handlers
  glutDisplayFunc(display_cb_);
  glutKeyboardFunc(key_cb_);
  glutMouseFunc(mouse_cb_);
  glutReshapeFunc(reshape_cb_);
  glutMotionFunc(motion_cb_);
}

void
Plane::
loop()
{
  // There is no way to return from glutMainLoop() directly; the
  // C++ exception mechanism is used instead.
  try
  {
    glutMainLoop();
  }
  catch (int)
  {
  }
}

void
Plane::
key_cb_(unsigned char key, int x, int y)
{
  _this->key_cb( key, x, _this->_height - y );
}

void
Plane::
mouse_cb_( int button, int state, int x, int y)
{
  _this->mouse_cb( button, state, x, _this->_height - y );
}

void
Plane::
display_cb_(void) 
{
  _this->display_cb();
}

void
Plane::
reshape_cb_(int width, int height)
{
  glViewport( 0, 0, width, height );
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  // note backwards y
  gluOrtho2D( 0, width, 0, height );

  // Call the subclass version.
  _this->reshape_cb( width, height );

  // cache the new width/height
  // do this after the call to reshape_cb so that function has access
  // to the old sizes
  _this->_width = width;
  _this->_height = height;

}

void
Plane::
motion_cb_( int x, int y )
{
  // Call the subclass version.
  _this->motion_cb(x, _this->_height - y);
}

void
Plane::
clear_window()
{
  // clear the display
  glClear(GL_COLOR_BUFFER_BIT);
}

void
Plane::
set_point_size( float x )
{
  glPointSize(x);
}

void
Plane::
set_line_size( float x )
{
  glLineWidth(x);
}

void
Plane::
set_color( Color const &color )
{
  glColor3fv( color.x );
}

void
Plane::
write_number( int n, Point const &p )
{
  string s;
  s.to_string( n );
  glRasterPos2i( (int)rint(p.x), (int)rint(p.y) );
  size_t i;
  for ( i = 0; i != s.length(); i++)
  {
    glutBitmapCharacter(GLUT_BITMAP_8_BY_13, s[i]);
  }
}

void
Plane::
draw_point( Point const &p )
{
  glBegin(GL_POINTS);
  glVertex2f( p.x, p.y );
  glEnd();
}

void
Plane::
draw_points( /*std*/::vector<Point> const &v )
{
  glBegin(GL_POINTS);

  /*std*/::vector<Point>::const_iterator i;
  for ( i = v.begin(); i != v.end(); i++ )
  {
    glVertex2f( (*i).x, (*i).y );
  }

  glEnd();
}

void
Plane::
draw_line( Point const &a, Point const &b )
{
  glBegin(GL_LINES);
  glVertex2i( (int)rint(a.x), (int)rint(a.y) );
  glVertex2i( (int)rint(b.x), (int)rint(b.y) );
  glEnd();
}

void
Plane::
draw_lines( /*std*/::vector<Point> const &v, bool loop )
{
  if ( v.size() < 2 ) { return; }

  glBegin(GL_LINES);

  /*std*/::vector<Point>::const_iterator i;
  for ( i = v.begin(); i != v.end() - 1; i++ )
  {
    glVertex2i( (int)rint((*i).x), (int)rint((*i).y) );
    glVertex2i( (int)rint((*(i+1)).x), (int)rint((*(i+1)).y) );
  }

  if ( loop )
  {
    // draw the final line making a loop
    i = v.end() - 1;
    glVertex2i( (int)rint((*i).x), (int)rint((*i).y) );
    i = v.begin();
    glVertex2i( (int)rint((*i).x), (int)rint((*i).y) );
  }

  glEnd();
}

void
Plane::
flush()
{
  glFlush();
}

void
Plane::
set_background_color( Color const &x )
{
  // set background to black
  glClearColor( x.x[0], x.x[1], x.x[2], 1.0 );
  glClear(GL_COLOR_BUFFER_BIT);
  display_cb_();
  glFlush();
}

} // namespace curve
