// Copyright (C) 2000 GANG www.gang.umass.edu

#ifndef Plane_INCLUDED
#define Plane_INCLUDED

#include <vector.h>
#include "Use.h"
#include "Vector2.H"

namespace curve {
using namespace math;

class Plane
{

public:

  Plane(
    int width, int height,
    int argc, char *argv[] );

  virtual ~Plane();

  void clear_window();

  void set_point_size( float );
  void set_line_size( float );

  struct Color {
    Color( float r, float g, float b )
      { x[0] = r; x[1] = g; x[2] = b; }
    float x[3];
  };

  typedef double real;
  typedef vector2<real> Point;

  void set_color( Color const & );
  void flush();
  void write_number( int n, Point const & );

  void draw_point( Point const & );
  void draw_points( /*std*/::vector<Point> const & );
  void draw_line( Point const &, Point const & );
  void draw_lines( /*std*/::vector<Point> const &, bool loop );

  void loop();

  static void set_background_color( Color const & );

protected:
  virtual inline void reshape_cb( int width, int height );
  virtual inline void key_cb(unsigned char key, int x, int y);
  virtual inline void mouse_cb( int button, int state, int x, int y);
  virtual inline void display_cb();
  virtual inline void motion_cb( int x, int y );

  float _width;
  float _height;

  Plane( Plane const & );
  Plane &operator=( Plane const & );

  int _window;

private:

  void init(
    int width, int height,
    int argc, char *argv[] );

  static void reshape_cb_(int width, int height);
  static void key_cb_(unsigned char key, int x, int y);
  static void mouse_cb_( int button, int state, int x, int y);
  static void display_cb_() ;
  static void motion_cb_( int x, int y );

  static Plane *_this;

};


void
Plane::
reshape_cb( int width, int height )
{
  USE(width);
  USE(height);
}

void
Plane::
key_cb(unsigned char key, int x, int y)
{
  USE(key); USE(x); USE(y);
}

void
Plane::
mouse_cb( int button, int state, int x, int y)
{
  USE(button); USE(state); USE(x); USE(y);
}

void
Plane::
display_cb()
{
}

void
Plane::
motion_cb( int x, int y )
{
  USE(x); USE(y);
}

} // namespace curve

#endif // !Plane_INCLUDED
