// Copyright (C) 2000 GANG http://www.gang.umass.edu

#include "MathReal.H"
#include "curve.H"
#include "Polygon.H"

namespace curve {

void
Polygon::
compute() throw ( Exception )
{
  compute_turning_number();
  jiggle();
}


void
Polygon::
compute_turning_number() throw ( Exception )
{
  /*
    Computes and returns the turning number.

    ALGORITHM:
                        C
                         ^
                        /
                       /
                      / angle
        A --->------ B - - - - - X

    The curve is really a polygon. In the diagram, the curve goes from
    A to B to C. Extend AB to X, Let theta_B be the oriented angle XBC.
    The turning number is the sum of the thetas at all the vertices,
    divided by 2 pi.

    *** TO DO: Test this for the case of very sharp angles (i.e. 2 pi)
  */

  if ( _polygon.size() < 3 )
  {
    throw ERR_POLYGON;
  }

  real a = 0.0;
  uint i;
  for ( i = 0; i < _polygon.size(); i++ )
  {
    uint j0 = (i == 0 ? _polygon.size()-1 : i-1);
    uint j1 = i;
    uint j2 = (i == _polygon.size()-1 ? 0 : i+1);
    Point const &a0 = _polygon[j0];
    Point const &a1 = _polygon[j1];
    Point const &a2 = _polygon[j2];
    real b = Point::angle(a1-a0, a2-a1);
    a += b;
  }

  _turning_number = (int)rint(a/(2.0*MathReal::pi));
}


void
Polygon::
jiggle()
{
  /*
     Jiggle the points
     This is almost certain to prevent two special cases:
       coincident verticies, and _polygons on lines
     (Thanks to Adam for the idea)

     *** TO DO: should jiggle only once, check with a flag
  */

  real epsilon = 1e-3;

  uint i;
  for ( i = 0; i < _polygon.size(); i++ )
  {
    _polygon[i].x += epsilon * ( drand48() - 0.5 );
    _polygon[i].y += epsilon * ( drand48() - 0.5 );
  }
}

} // namespace curve
